# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: datasync.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='datasync.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0e\x64\x61tasync.proto\" \n\x0f\x44\x61taSyncRequest\x12\r\n\x05value\x18\x01 \x01(\t\"!\n\x10\x44\x61taSyncResponse\x12\r\n\x05value\x18\x01 \x01(\t2=\n\x08\x44\x61taSync\x12\x31\n\x08\x44\x61taSync\x12\x10.DataSyncRequest\x1a\x11.DataSyncResponse\"\x00\x62\x06proto3'
)




_DATASYNCREQUEST = _descriptor.Descriptor(
  name='DataSyncRequest',
  full_name='DataSyncRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='DataSyncRequest.value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=18,
  serialized_end=50,
)


_DATASYNCRESPONSE = _descriptor.Descriptor(
  name='DataSyncResponse',
  full_name='DataSyncResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='DataSyncResponse.value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=52,
  serialized_end=85,
)

DESCRIPTOR.message_types_by_name['DataSyncRequest'] = _DATASYNCREQUEST
DESCRIPTOR.message_types_by_name['DataSyncResponse'] = _DATASYNCRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DataSyncRequest = _reflection.GeneratedProtocolMessageType('DataSyncRequest', (_message.Message,), {
  'DESCRIPTOR' : _DATASYNCREQUEST,
  '__module__' : 'datasync_pb2'
  # @@protoc_insertion_point(class_scope:DataSyncRequest)
  })
_sym_db.RegisterMessage(DataSyncRequest)

DataSyncResponse = _reflection.GeneratedProtocolMessageType('DataSyncResponse', (_message.Message,), {
  'DESCRIPTOR' : _DATASYNCRESPONSE,
  '__module__' : 'datasync_pb2'
  # @@protoc_insertion_point(class_scope:DataSyncResponse)
  })
_sym_db.RegisterMessage(DataSyncResponse)



_DATASYNC = _descriptor.ServiceDescriptor(
  name='DataSync',
  full_name='DataSync',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=87,
  serialized_end=148,
  methods=[
  _descriptor.MethodDescriptor(
    name='DataSync',
    full_name='DataSync.DataSync',
    index=0,
    containing_service=None,
    input_type=_DATASYNCREQUEST,
    output_type=_DATASYNCRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_DATASYNC)

DESCRIPTOR.services_by_name['DataSync'] = _DATASYNC

# @@protoc_insertion_point(module_scope)



import sys
import traceback


class MacValidateError(Exception):
    pass


class TokenMissingError(Exception):
    pass


def trace_detail_info(err):
    error_class = err.__class__.__name__  # 取得錯誤類型
    detail = err.args[0]  # 取得詳細內容
    _, _, tb = sys.exc_info()  # 取得Call Stack
    lastCallStack = traceback.extract_tb(tb)[-1]  # 取得Call Stack的最後一筆資料
    fileName = lastCallStack[0]  # 取得發生的檔案名稱
    lineNum = lastCallStack[1]  # 取得發生的行號
    funcName = lastCallStack[2]  # 取得發生的函數名稱
    errMsg = f"File \"{fileName}\", line {lineNum}, in {funcName}: [{error_class}] {detail}"
    return errMsg



return_code = {
    'Success': '4001',                  # API調用成功
    'SaveSuccess': '4002',              # 資料儲存成功
    'PredictSucess': '4003',            # 模型預測成功
    'ServiceRegisterSuccess': '4004',   # 服務註冊成功
    'TaskAssignSuccess': '4005',        # 排成建立成功
    'ServiceStart': '4006',             # 服務啟動成功
    'TrainFinished': '4007',            # 模型訓練完成
    'TrainInProgress': '4008',          # 模型訓練進行中

    'InvalidAPIKey': '0101',            # API key為空
    'InvalidInput': '0102',             # 輸入資料格式錯誤
    'MacInvalid': '0103',               # mac 無法取得
    'TimeStampInvalid': '0104',         # timestamp 無法取得
    'RequiredFieldsEmpty': '0105',      # 必要欄位為空值

    'ValidationFailed': '9903',         # mac 驗證失敗
    'ModelBuildFailed': '9904',         # 模型啟動失敗
    'PredictFailed': '9905',            # 模型預測失敗
    'ServiceRegisterFailed': '9906',    # 服務註冊失敗
    'ServiceStartFailed': '9908',       # 服務啟動失敗
    'TrainFailed': '9909',              # 模型訓練失敗
    'StopRetry': '9910',                # 終止重複執行
    'DatabaseError':'9923',            # DB Error
    'RedisConnectionError': '9924',     # Redis連線失敗
    'CodisConnectionError': '9925',     # Codis連線失敗

    'InidentifyInput': '2001',          # 無法識別資料
    'SystemError': '9999'               # 系統錯誤
}



import json
import pytz
import grpc
import decimal
import datetime

from . import datasync_pb2, datasync_pb2_grpc
from ..database.model_handler import ModelHandler


class DateEncoder(json.JSONEncoder):
    """
    自定义类，解决报错：
    TypeError: Object of type "datetime" is not JSON serializable
    """

    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.astimezone(pytz.UTC).strftime("%Y-%m-%d %H:%M:%S")
        elif isinstance(obj, datetime.date):
            return obj.astimezone(pytz.UTC).strftime("%Y-%m-%d")
        elif isinstance(obj, decimal.Decimal):
            return float(obj)
        else:
            return json.JSONEncoder.default(self, obj)


class DataSyncHandler:
    def __init__(
            self, 
            db_settings: dict, 
            datasync_config: dict,
            logger
        ) -> None:
        self.DB_NAME = db_settings.get('db_name')
        self.DATASYNC_DATASOURCE = datasync_config.get('datasource')
        self.DATASYNC_USE = datasync_config.get('datasync_use')
        self.logger = logger
        self.db_handler = ModelHandler(
            db=db_settings.get('db'), 
            table_info=db_settings.get('table_info'), 
            table_result=db_settings.get('table_result'),
            logger=logger
        )

        self.channel = grpc.insecure_channel(datasync_config.get('grpc_domain'))
        self.stub = datasync_pb2_grpc.DataSyncStub(self.channel)
        
        
    def datasync_func(self, data, pk_id=0):
        if self.DATASYNC_USE != 'true':
            return None

        sn = data.serial_number
        try:
            output = dict()
            output["data_source"] = self.DATASYNC_DATASOURCE
            output["schema"] = self.DB_NAME
            output["table"] = data.__tablename__
            output["payload"] = {x.name: getattr(data, x.name) for x in data.__table__.columns}
            output["pk_id"] = pk_id

            request = datasync_pb2.DataSyncRequest(value=json.dumps(output, cls=DateEncoder))
            response = self.stub.DataSync(request)
            response = json.loads(response.value)
            self.logger.info(
                f'[{sn}]DataSync {data.__tablename__} Result: {response["reason"][0]}')

        except grpc.RpcError as rpc_error:
            if rpc_error.code() == grpc.StatusCode.CANCELLED:
                self.logger.error(
                    f"[{sn}]DataSync Catch an exception: DataSync Status Canceled")
            elif rpc_error.code() == grpc.StatusCode.UNAVAILABLE:
                self.logger.error(
                    f"[{sn}]DataSync Catch an exception: DataSync Connection Unavaliable")

        except:
            self.logger.exception(f"[{sn}]DataSync Catch an exception:")


    def save_data_and_sync(self, data_type='info', pk_id=None, **data):
        datasync_content = self.db_handler.save_data(data_type, **data)

        if datasync_content:
            self.logger.info(f"[{data['serial_number']}]Datasync flow : {data_type}")
            if data_type == 'info':
                info_PKID = datasync_content.pk_id
                self.datasync_func(datasync_content, pk_id=info_PKID)
                return info_PKID
            else:
                self.datasync_func(datasync_content, pk_id=pk_id)
                return pk_id

